#ifndef __SEA_ANEMONE_H
#define __SEA_ANEMONE_H

#include "ShapeGenerator.h"
#include "interpolation.h"
#include "GlobalPhysics.h"
#include "Hair.h"

using namespace ci;

class SeaAnemone{
public:
    SeaAnemone();
	~SeaAnemone();

    void update(float time);
    void regenerate(float time);
    void draw();
    void drawDebug();

    //float scaleXZ;
    //float scaleY;

    float R; //on bottom
    float r; //on top
    float H; //height

    float mr; //middle radius
    float mrH; //middle radius height

    //number of divisions horizontally/vertically
    int radialSegments; 
    int heightSegments;

    int numWaves; //number of waves on bottom
    float waveD; //wave size

    float soleClamp; //from that height generate waves

    int hairInLine; //number of hair in one circle
    int hairLines; //number of hair circles
    float hairR; //hair R (bottom)
    float hairH; //hair H
    bool hairCylindrical; //cylinder or cone

    bool firstRun;

    float bendAngleDeg; //how much should it bend
    Vec3f bendAxis;

    float verticalScale; //flatten top scale

    //drawing only:
    bool viewNormals;
    bool viewTriangles;
    bool viewModel;
	bool viewHair;
    float normalLength;

    gl::Texture gradientColor;

    std::vector<Vec3f> colors;
    std::vector<float> colorStartsAt;

    std::vector<float> heights;
    std::vector<float> heightsHair;
	float basalHeight;

    std::list<modGeo::interpolation> curves;

	float mLastTime;

	GlobalPhysics m_physics;
	std::vector<Hair> m_hairSet;

	ci::Vec3f mBallPos;

	void perturbateHair(float dt);

    void drawBasalDisc(); //draw "the sole" of the sea anemone
    void drawCnidocytes(); //draw "the hair"

private:
    //time variable might be used later for animation
    void generateBasalDisc(float time);
    void generateCnidocytes(float time, bool create);

    void buildHairLine(Vec3f offset, Quaternion<float> rotation, Quatf rotation2normal, float radius, float hairCount, bool create, float angleStart);
    void buildSingleHair(Vec3f offset, Quaternion<float> rotation, bool create);
    void buildSingleHairCurve(const Hair &hair, modGeo::interpolation& curve);
    void buildSingleHairModel(const modGeo::interpolation& curve);
    std::pair<Vec3f,Quatf> getHeadPositionAndRotation();

    std::vector<Vec3f> vertices;
    std::vector<uint32_t> triangles;
    std::vector<Vec3f> normals;
    std::vector<Vec2f> texCoords;

	std::vector<Vec3f> verticesHair;
    std::vector<uint32_t> trianglesHair;
    std::vector<Vec3f> normalsHair;
    std::vector<Vec2f> texCoordsHair;

    void recalculateTriangleNormals();
    std::vector<Vec3f> triangleNormals;

    TriMesh mesh;
	TriMesh hairMesh;

	gl::VboMesh::Layout layout;

    ShapeGenerator generator;
	ShapeGenerator hairGenerator;
};

#endif