#ifndef __SHAPE_GENERATOR_H
#define __SHAPE_GENERATOR_H

using namespace ci;

class ShapeGenerator{
public:
    ShapeGenerator(std::vector<Vec3f>& vertices, std::vector<Vec3f>& normals, std::vector<uint32_t>& triangles, 
		std::vector<Vec2f>& texcoordsRef, std::vector<float>& heights);

    //cylinder generation

    /**
        builds cylinder given center bottom point, height and radius
        radialsegments / heightsegments - density of segments
        rotation of a cylinder given by quaternion
        if you want cylinder to have "waves", give number of waves and wave height
    */
    void buildCylinder(Vec3f center, float H, float radius, int radialSegments, int heightSegments, 
        Quaternion<float> rotation, int numWaves, float waveD);
    
    /**
        help function for cylinder building;
        segments = radial segments, center = center of a single ring, radius, rotation - as exaplained above
        trianglesB - bool, set to true if it's not the first ring (there are already some points to connect to in order to create a triangle mesh)
        waves - do you want waves, numwaves and waveD as above
        wavesH - scaling wave (waves on the bottom should be bigger than the ones higher
    */
    void buildCylinderRing(int segments, Vec3f center, float radius, Quaternion<float> rotation, float v, 
        bool trianglesB, bool waves, int numWaves, float waveD, float wavesH=0.f, float height=-1.f);

    /**
        help function for cylinder building
        generates a cap on top/bottom to close the mesh
        reverse = should the cap be on bottom (normal direction different)
        rest of the parameters as explained above
    */
    void buildCap(int segments, Vec3f center, float radius, Quaternion<float> rotation, bool reverse, 
        bool waves, int numWaves, float waveD, float wavesH=0.f, float height=-1.f);

    
    //cone generation
    /**
        cone build method, builds cone height H, bottom center = center
        radiuses R_start, R_end, if R_end != 0 is't a cone without pointy top
    */
    void buildCone(Vec3f center, float H, float R_start, float R_end, int radialSegments, 
        int heightSegments, Quaternion<float> rotation, Vec3f h_axis);

    /**
        helper for cone building, 
        new parameter slope - for normal calculations,
        slope.x = R_end - R_start,
        slope.y = H
        (normalized)
    */
    void buildConeRing(int segments, Vec3f center, float radius, Quaternion<float> rotation, float v, 
        bool buildTriangles, Vec2f slope, float height = -1.f);


    //sphere generation
    /**
        builds a sphere given radius and densities, 
        center = bottom center point (I know I'm lazy)
    */
    void buildSphere(Vec3f center, float radius, int radialSegments, int heightSegments, float height = -1.f);
    /**
        help for sphere building, nothing new here
    */
    void buildSphereRing(int segments, Vec3f center, float radius, Quaternion<float> rotation, float v, bool buildTriangles, float height = -1.f);

    /**
        builds a flatten in Y-axis sphere
        verticalScale in (0,1)
    */
    void buildFlatSphere(Vec3f center, float radius, Quaternion<float> rotation, 
        int radialSegments, int heightSegments, float verticalScale, float height = -1.f);

    /**
        Adds triangle indices to triangles list
    */
    void addTriangle(int a, int b, int c);

private:
    std::vector<Vec3f>& vertices;
    std::vector<Vec3f>& normals;
    std::vector<uint32_t>& triangles;
    std::vector<Vec2f>& texCoords;
    std::vector<float>& heights;
};

#endif //__SHAPE_GENERATOR_H