#include "PrecompiledHeaders.h"

#include "CallbackMenu.h"

namespace cinder { 
    namespace params {

        CallbackMenuRef CallbackMenu::create( const std::string &title, const Vec2i &size, const ColorA &color )
        {
            return shared_ptr<CallbackMenu>( new CallbackMenu( title, size, color ) );
        }

        CallbackMenuRef CallbackMenu::create( cinder::app::WindowRef window, const std::string &title, const Vec2i &size, const ColorA &color )
        {
            return shared_ptr<CallbackMenu>( new CallbackMenu( window, title, size, color ) );
        }

        void CallbackMenu::addParam( const std::string &name, bool *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_BOOLCPP, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, float *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_FLOAT, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, double *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_DOUBLE, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, int32_t *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_INT32, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, Vec3f *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_DIR3F, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, Quatf *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_QUAT4F, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, Color *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_COLOR3F, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, ColorA *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_COLOR4F, optionsStr, readOnly );
        } 

        void CallbackMenu::addParam( const std::string &name, std::string *param, const std::string &optionsStr, bool readOnly )
        {
            implAddParam( name, param, TW_TYPE_STDSTRING, optionsStr, readOnly );
        }

        void CallbackMenu::addParam( const std::string &name, const std::vector<std::string> &enumNames, int *param, const std::string &optionsStr, bool readOnly )
        {
            TwSetCurrentWindow( mTwWindowId );

            TwEnumVal *ev = new TwEnumVal[enumNames.size()];
            for( size_t v = 0; v < enumNames.size(); ++v ) {
                ev[v].Value = v;
                ev[v].Label = const_cast<char*>( enumNames[v].c_str() );
            }

            TwType evType = TwDefineEnum( (name + "EnumType").c_str(), ev, enumNames.size() );

            if( readOnly )
                TwAddVarRO( mBar.get(), name.c_str(), evType, param, optionsStr.c_str() );
            else
                TwAddVarRW( mBar.get(), name.c_str(), evType, param, optionsStr.c_str() );

            delete [] ev;
        }

        template <class T>
        struct Accessors {
            Accessors( std::function<void(T)> setter, std::function<T()> getter )
                : mSetter( setter )
                , mGetter( getter ) { }
            std::function<void(T)> mSetter;
            std::function<T()> mGetter;
        };

        template <class T>
        void TW_CALL setCallbackImpl(const void *value, void *clientData)
        {
            Accessors<T> *twc = reinterpret_cast<Accessors<T>*>( clientData );
            twc->mSetter( *(const T *)value );
        }

        template <class T>
        void TW_CALL getCallbackImpl(void *value, void *clientData)
        {
            Accessors<T> *twc = reinterpret_cast<Accessors<T>*>( clientData );
            *(T *)value = twc->mGetter();
        }

        template <class T>
        void CallbackMenu::implAddParamCb( const std::string &name, int type, const std::string &optionsStr, const std::function<void (T)> &setter, const std::function<T ()> &getter )
        {
            TwSetCurrentWindow( mTwWindowId );

            auto callbackPtr = std::make_shared<Accessors<T>>( setter, getter );
            mStoredCallbacks.push_back( callbackPtr );

            TwAddVarCB( mBar.get(), name.c_str(), (TwType) type, setCallbackImpl<T>, getCallbackImpl<T>, (void *)callbackPtr.get(), optionsStr.c_str() );
        }


        void CallbackMenu::addParam( const std::string &name, const std::function<void (bool)> &setter, const std::function<bool ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<bool>(name, TW_TYPE_BOOLCPP, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (float)> &setter, const std::function<float ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<float>(name, TW_TYPE_FLOAT, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (double)> &setter, const std::function<double ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<double>(name, TW_TYPE_DOUBLE, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (int32_t)> &setter, const std::function<int32_t ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<int32_t>(name, TW_TYPE_INT32, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (Vec3f)> &setter, const std::function<Vec3f ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<Vec3f>(name, TW_TYPE_DIR3F, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (Quatf)> &setter, const std::function<Quatf ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<Quatf>(name, TW_TYPE_QUAT4F, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (Color)> &setter, const std::function<Color ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<Color>(name, TW_TYPE_COLOR3F, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (ColorA)> &setter, const std::function<ColorA ()> &getter, const std::string &optionsStr )
        {
            implAddParamCb<ColorA>(name, TW_TYPE_COLOR4F, optionsStr, setter, getter );
        }

        void CallbackMenu::addParam( const std::string &name, const std::function<void (std::string)> &setter, const std::function<std::string ()> &getter, const std::string &optionsStr  )
        {
            implAddParamCb<std::string>(name, TW_TYPE_STDSTRING, optionsStr, setter, getter );
        }


    }}