#include "PrecompiledHeaders.h"

#include "GlobalPhysics.h"

GlobalPhysics::GlobalPhysics() {
	mBroadphase = new btDbvtBroadphase();
	mCollisionConfiguration = new btDefaultCollisionConfiguration();
    mDispatcher = new btCollisionDispatcher(mCollisionConfiguration);
	mSolver = new btSequentialImpulseConstraintSolver;
	mDynamicsWorld = new btDiscreteDynamicsWorld(mDispatcher,mBroadphase,mSolver,mCollisionConfiguration);
	mDynamicsWorld->setGravity(btVector3(0,-25,0));

	mBallShape = new btSphereShape(30);
    mBallMotionState = new btDefaultMotionState(btTransform(btQuaternion(0,0,0,1),btVector3(150,300,0)));
    mBallMass = 2000;
    mBallShape->calculateLocalInertia(mBallMass,mBallIntertia);
    btRigidBody::btRigidBodyConstructionInfo lBallRigidBodyCI(mBallMass,mBallMotionState,mBallShape,mBallIntertia);
    mBallRigidBody = new btRigidBody(lBallRigidBodyCI);
    mDynamicsWorld->addRigidBody(mBallRigidBody);
}


void GlobalPhysics::ballReset() {
	mBallRigidBody->setWorldTransform(btTransform(btQuaternion(0,0,0,1),btVector3(150,300,0)));
	mBallRigidBody->setLinearVelocity(btVector3(0,0,0));
    mBallRigidBody->applyCentralImpulse(btVector3(-200000,50,0));
}

GlobalPhysics::~GlobalPhysics() {
	delete mDynamicsWorld;
	delete mSolver;
	delete mDispatcher;
	delete mCollisionConfiguration;
	delete mBroadphase;
}

void GlobalPhysics::addRigidBody(btRigidBody *rigidBody) {
	mDynamicsWorld->addRigidBody(rigidBody);
}

void GlobalPhysics::removeRigidBody(btRigidBody *rigidBody) {
	mDynamicsWorld->removeRigidBody(rigidBody);
}

void GlobalPhysics::addConstraint(btTypedConstraint *constraint) {
	mDynamicsWorld->addConstraint(constraint);
}

void GlobalPhysics::removeConstraint(btTypedConstraint *constraint) {
	mDynamicsWorld->removeConstraint(constraint);
}

void GlobalPhysics::advance(float dt) {
	mDynamicsWorld->stepSimulation(dt,10);
}

