#include "PrecompiledHeaders.h"
#include "SeaAnemone.h"

SeaAnemone::SeaAnemone() : generator(vertices, normals, triangles, texCoords, heights), 
	hairGenerator(verticesHair, normalsHair, trianglesHair, texCoordsHair, heightsHair) {
    r = 70.f;
    R = 100.f;
    H = 200.f;

    mr = 60.f;
    mrH = 0.9f;

    heightSegments = 20;
    radialSegments = 50;

    numWaves = 10;
    waveD = 0.1f;

    soleClamp = 0.2f;

    bendAngleDeg = 0.f;
    bendAxis = Vec3f::zAxis();

    viewNormals = false;
    viewTriangles = false;
	viewHair = true;
    normalLength = 10.f;

    verticalScale = 0.25f;

    hairInLine = 20;
	hairLines = 4;
    hairR = 5.f;
    hairH = 10.f;
    hairCylindrical = false;

	colors.push_back(Vec3f(80.0f/255.0f, 33.0f/255.0f, 20.0f/255.0f));
	colors.push_back(Vec3f(173.0f/255.0f, 56.0f/255.0f, 65.0f/255.0f));
	colors.push_back(Vec3f(238.0f/255.0f, 128.0f/255.0f, 157.0f/255.0f));
	colors.push_back(Vec3f(249.0f/255.0f, 152.0f/255.0f, 92.0f/255.0f));
	colors.push_back(Vec3f(247.0f/255.0f, 204.0f/255.0f, 222.0f/255.0f));
	colors.push_back(Vec3f(255.0f/255.0f, 255.0f/255.0f, 255.0f/255.0f));

	colorStartsAt.push_back(0.0f);
	colorStartsAt.push_back(0.35f);
	colorStartsAt.push_back(0.63f);
	colorStartsAt.push_back(0.78f);
	colorStartsAt.push_back(0.88f);
	colorStartsAt.push_back(1.0f);

    layout.setStaticPositions();
    layout.setStaticNormals();
    layout.setStaticTexCoords2d();
    layout.setStaticIndices();
    
	firstRun = true;

	mLastTime = 0.f;
}

SeaAnemone::~SeaAnemone() { }

void SeaAnemone::draw(){
    //add textures, gradients, fluorescence, alpha
    drawBasalDisc();
    drawCnidocytes();
}

void SeaAnemone::drawBasalDisc(){
    if(viewModel){
        gl::draw(mesh);
    }
}

void SeaAnemone::drawDebug(){
    if(viewNormals){
        //recalculate only when mesh changes
        recalculateTriangleNormals();

        size_t polycount = mesh.getNumTriangles();        
        Vec3f v0, v1, v2;
        for(size_t i=0; i<polycount; ++i)
        {    
            mesh.getTriangleVertices(i, &v0, &v1, &v2);
            gl::color(Colorf(1.f, 1.f, 1.f));
            Vec3f center = (v0+v1+v2)/3;
            gl::drawVector(center, center + normalLength*triangleNormals[i]);
        }
		size_t hairpolycount = hairMesh.getNumTriangles();        
        for(size_t i=0; i<hairpolycount; ++i)
        {    
            hairMesh.getTriangleVertices(i, &v0, &v1, &v2);
            gl::color(Colorf(1.f, 1.f, 1.f));
            Vec3f center = (v0+v1+v2)/3;
            gl::drawVector(center, center + normalLength*triangleNormals[polycount+i]);
        }
    }
    if(viewTriangles){
        gl::enableWireframe();
        gl::color(Colorf(0.96f, 0.48f, 0.51f));
		gl::draw(mesh);
		gl::draw(hairMesh);
		gl::disableWireframe();
    }
}

void SeaAnemone::drawCnidocytes(){
	if(viewHair) {
		gl::draw(hairMesh);
	}
}

// odtworz od poczatku caly model i symulacje fizyczna
void SeaAnemone::regenerate(float time){
	firstRun = false;

    // clear models
    mesh.clear();
	hairMesh.clear();

	// clear generators lists
    triangles.clear();
    vertices.clear();
    normals.clear();
    texCoords.clear();
	heights.clear();
	    
	trianglesHair.clear();
    verticesHair.clear();
    normalsHair.clear();
    texCoordsHair.clear();
	heightsHair.clear();

	// regenerate data
    generateBasalDisc(time);
	float height = *std::max_element(heights.begin(), heights.end());
	basalHeight = height;

	generateCnidocytes(time,true);
	//find overall height, assumption: generation should be called once not on every draw
	float heightH = *std::max_element(heightsHair.begin(), heightsHair.end());
	for(unsigned int i = 0; i < heights.size(); ++i){
		heights[i] /= heightH;
	}
	for(unsigned int i = 0; i < heightsHair.size(); ++i){
		heightsHair[i] /= heightH;
	}
	

	// regenerate models
    mesh.appendVertices(&vertices[0], vertices.size());
    mesh.appendIndices(&triangles[0], triangles.size());
    mesh.appendNormals(&normals[0], normals.size());
    mesh.appendTexCoords(&texCoords[0], texCoords.size());

	hairMesh.appendVertices(&verticesHair[0], verticesHair.size());
    hairMesh.appendIndices(&trianglesHair[0], trianglesHair.size());
    hairMesh.appendNormals(&normalsHair[0], normalsHair.size());
    hairMesh.appendTexCoords(&texCoordsHair[0], texCoordsHair.size());
}

// zrob tylko to co trzeba
void SeaAnemone::update(float time){
	firstRun = false;

	float dt = time - mLastTime;
	mLastTime = time;

    hairMesh.clear();

    trianglesHair.clear();
    verticesHair.clear();
    normalsHair.clear();
    texCoordsHair.clear();
	heightsHair.clear();

	generateCnidocytes(time,false);

	//find overall height, assumption: generation should be called once not on every draw
	float heightH = *std::max_element(heightsHair.begin(), heightsHair.end());
	for(unsigned int i = 0; i < heightsHair.size(); ++i){
		heightsHair[i] /= heightH;
	}

	hairMesh.appendVertices(&verticesHair[0], verticesHair.size());
    hairMesh.appendIndices(&trianglesHair[0], trianglesHair.size());
    hairMesh.appendNormals(&normalsHair[0], normalsHair.size());
    hairMesh.appendTexCoords(&texCoordsHair[0], texCoordsHair.size());

	btTransform trans;
    m_physics.mBallRigidBody->getMotionState()->getWorldTransform(trans);
    mBallPos = ci::Vec3f(trans.getOrigin().getX(),trans.getOrigin().getY(),trans.getOrigin().getZ());

	perturbateHair(time);
	m_physics.advance(dt);
}

void SeaAnemone::generateBasalDisc(float time){
    //todo:
    //waves/non-waves parts should connect smoothly
    float dheight = H / heightSegments;


    float bendAngleRad= toRadians(bendAngleDeg);
    float bendRadius = H / bendAngleRad;
    float dangle = bendAngleRad / heightSegments;
    Vec3f startOffset = Vec3f(bendRadius, 0.0f, 0.0f);

    Vec3f centrePos = Vec3f::zero();
    Quaternion<float> rotation = Quaternion<float>();

    for (int i = 0; i <= heightSegments; i++)
    {
        if(bendAngleDeg!=0.0f){
            centrePos.x = cos(dangle * i);
            centrePos.y = sin(dangle * i);

            rotation.set(bendAxis, dangle * i);

            centrePos *= bendRadius;
            centrePos -= startOffset;
        }
        else{
            centrePos = Vec3f::yAxis() * dheight * i;
        }
		
        float current_h = (float)i/heightSegments;

        float radius;
        if(mrH >= current_h){
            float dh = H * mrH / dheight;
            radius = lerp(R, mr, (float)i / dh);
        }
        else{
            float dh = H * (1.f - mrH) / dheight;
            radius = lerp(mr, r, 1.f - (float)(heightSegments -i) / dh);
        }

        float v = (float)i / heightSegments;

        //use different waveH argument for nice and smooth change?
        generator.buildCylinderRing(radialSegments, centrePos, radius, rotation, v, i > 0, current_h<=soleClamp, 
			numWaves, waveD, soleClamp - current_h);
		

        if (i==0){
            generator.buildCap(radialSegments, centrePos, radius, rotation, true, current_h<=soleClamp, numWaves, waveD, soleClamp - current_h);
		}
        else if (i==heightSegments){
            generator.buildFlatSphere(centrePos, radius, rotation, radialSegments, heightSegments, verticalScale);
		}
    }
}

void SeaAnemone::generateCnidocytes(float time, bool create){
	// znajduje pozycje i rotacje konca nogi (bez wieczka)
	Vec3f centrePos;
	Quatf rotation;
	std::tie(centrePos,rotation) = getHeadPositionAndRotation();

	// poczatek wlosa na powierzchni basal disc (konkretnie flatSphere)
	// poczatek wlosa ustawiony prostopadle do powierzchni (po normalnej)
	// kolejne *rzedy* wlosow ustawione co staly krok katowy od "rownika" do "bieguna" (na splaszczonej kuli)
	// ilosc wlosow w kolejnych rzedach = hairInLine / (1+(i-1)*0.5)
	// kolejne wlosy w rzedzie co staly krok katowy
	// pierwszy wlos w rzedzie przesuniety o M_PI/6*pow(i,2) dla mniej regularniego wygladu

    float angleInc = (float)(M_PI / 2.f / (hairLines+2));
	
	if(create) {
		m_hairSet.clear();
		// znajduje pozycje i normalna do basal disc (konkretnie flatSphere)
		for(int i = 1; i <= hairLines; i++) {
			float capy = sin(angleInc * i) * r * verticalScale;
			float capx = cos(angleInc * i) * r;
			Vec3f capdalfa(0.f, cos(angleInc*i)*r*verticalScale, -sin(angleInc*i)*r);
			Vec3f capdz(1.f, 0.f, 0.f);
			Vec3f capnormal = -cross(capdalfa,capdz).normalized();
			Quatf rotation2normal(Vec3f(0.f, 1.f, 0.f), capnormal);

			buildHairLine(centrePos + rotation * Vec3f(0.0f, capy + r * verticalScale, 0.0f),rotation,rotation2normal,capx,hairInLine / (1+(i-1)*0.5f), create, (float)(M_PI/6*pow(i,2)));
		}
	}
	// tutaj tworzymy krzywe interpolacyjne i model
	for(const auto &hair : m_hairSet) {
		modGeo::interpolation curve;
		buildSingleHairCurve(hair, curve);
		buildSingleHairModel(curve);
	}
}

void SeaAnemone::buildHairLine(Vec3f offset, Quaternion<float> rotation, Quatf rotation2normal, float radius, float hairCount, bool create, float angleStart = 0.f){
    for (int i = 0; i < hairCount; i++) {
        float yAngle = angleStart + (float)(M_PI * 2.f * i) / hairCount;
        Quaternion<float> radialRotation(Vec3f::yAxis(), yAngle);
        Vec3f position = offset + radialRotation * rotation * Vec3f::zAxis() * radius;

        buildSingleHair(position, rotation2normal * radialRotation * rotation, create);
    }
}

void SeaAnemone::buildSingleHair(Vec3f offset, Quaternion<float> rotation, bool create) {
	btVector3 v3(offset.x, offset.y, offset.z);
	btQuaternion q4(rotation.v.x, rotation.v.y, rotation.v.z, rotation.w);
	m_hairSet.push_back(Hair(v3,q4,&m_physics,hairR,hairH));
}

void SeaAnemone::buildSingleHairCurve(const Hair &hair, modGeo::interpolation& curve) {
	// pobierz pooenia wzw acucha kinematycznego z symulacji fizycznych
	std::vector<Vec3f> pts;
	for(unsigned int i = 0; i < hair.m_segments.size(); i++) {
		btVector3 v = hair.m_segments[i]->getWorldTransform().getOrigin();
		pts.push_back(Vec3f(v.x(),v.y(),v.z()));
	}
		
	
	// wstaw pooenia do krzywej interpolacyjnej
	Vec3f direction = pts[1]-pts[0];
	curve.addInterPoint(pts[0]-direction*0.5f);
	for(unsigned int i = 0; i < hair.m_segments.size(); i++) {
		curve.addInterPoint(pts[i]);
	}

	// Ustawia styczna dla pierwszego punktu krzywej.
	// Krzywa powinna wyjsc styczna do tego wektora, czyli
	// w miare prostopadla do powierzchni.
	direction.normalize();
	curve.setFirstTangent(direction);

	// przelicz krzyw interpolacyjn
	curve.updateControlPoints();
}

// generowanie modelu z krzywej
void SeaAnemone::buildSingleHairModel(const modGeo::interpolation& curve) {
	// ograniczenie iloci segmentw, na podstawie iloci segmentw 'nogi' ukwiau
	int hairHeightSegments = int(heightSegments * 1.0);

    float rend = hairR / 2.f;
    Vec2f slope = Vec2f(rend - hairR, H);
    slope.normalize();

    for (int i = 0; i <= hairHeightSegments; i++) {
		Vec3f centrePos = curve.deCasteljauUniform(i * 1.0 / hairHeightSegments);
		Vec3f tangent = curve.tangent(i * 1.0 / hairHeightSegments);
		Quatf localRotation(Vec3f::yAxis(), tangent);
		Quatf totalRotation = localRotation;
        
        float radius = hairR;
        float v = (float)i / hairHeightSegments;

		int hairRadialSegments = int(radialSegments * 0.3);
		const float hairLength = 100.f;
		float lHeight = basalHeight + hairLength*float(i)/float(hairHeightSegments);
        if(hairCylindrical) {
            hairGenerator.buildCylinderRing(hairRadialSegments, centrePos, radius, totalRotation, v, i > 0, false, 0,0,0,lHeight);
            if (i == 0) {
                hairGenerator.buildCap(hairRadialSegments, centrePos, radius, totalRotation, true, false, 0,0,0,lHeight);
			} else if (i == hairHeightSegments) {
                hairGenerator.buildFlatSphere(centrePos, radius, totalRotation, hairRadialSegments, hairHeightSegments, 2.f*verticalScale, lHeight);
			}
        } else {
            float rad = lerp(radius, rend, (float)i / hairHeightSegments);
            hairGenerator.buildConeRing(hairRadialSegments, centrePos, rad, totalRotation, v, i > 0, slope, lHeight);
            if (i == hairHeightSegments) {
                hairGenerator.buildSphere(centrePos, rend+hairR/5.f, hairRadialSegments, hairHeightSegments, lHeight);
			}
        }
    }

}

void SeaAnemone::perturbateHair(float t) {
	for(const auto &hair : m_hairSet) {
		for(const auto &node : hair.m_segments) {
			btVector3 p = node->getWorldTransform().getOrigin();
			float x = p.x()/100.f;
			float y = p.y()/100.f; 
			float z = p.z()/100.f;
			node->applyCentralForce(400.f*btVector3(
				pow(sin(x)+cos(t*0.3f),2.f)	*sin(t*0.2f),
				sin(x+z*x)					*cos(t*0.14f),
				cos(z+x*y)					*cos(t*0.4f)
				));
		}
	}
}

std::pair<Vec3f,Quatf> SeaAnemone::getHeadPositionAndRotation() {
	float dheight = H / heightSegments;
    float bendAngleRad= toRadians(bendAngleDeg);
    float bendRadius = H / bendAngleRad;
    float dangle = bendAngleRad / heightSegments;
    Vec3f startOffset = Vec3f(bendRadius, 0.0f, 0.0f);
	Vec3f centrePos = Vec3f::zero();
    Quaternion<float> rotation = Quaternion<float>();

    for (int i = 0; i <= heightSegments; i++) {
        if(bendAngleDeg!=0.0f){
            centrePos.x = cos(dangle * i);
            centrePos.y = sin(dangle * i);

            rotation.set(bendAxis, dangle * i);

            centrePos *= bendRadius;
            centrePos -= startOffset;
        }
        else {
            centrePos = Vec3f::yAxis() * dheight * i;
        }
	}
	return std::pair<Vec3f,Quatf>(centrePos,rotation);
}

void SeaAnemone::recalculateTriangleNormals(){
    triangleNormals.clear();
    size_t polycount = mesh.getNumTriangles();
    for(size_t i=0; i<polycount; ++i)
    {
        uint32_t index0 = triangles[i * 3];
        uint32_t index1 = triangles[i * 3 + 1];
        uint32_t index2 = triangles[i * 3 + 2];

        Vec3f v0 = normals[ index0 ];
        Vec3f v1 = normals[ index1 ];
        Vec3f v2 = normals[ index2 ];
        triangleNormals.push_back((v0+v1+v2)/3.f);
    }
	size_t hairpolycount = hairMesh.getNumTriangles();
	for(size_t i=0; i<hairpolycount; ++i)
    {
        uint32_t index0 = trianglesHair[i * 3];
        uint32_t index1 = trianglesHair[i * 3 + 1];
        uint32_t index2 = trianglesHair[i * 3 + 2];

        Vec3f v0 = normalsHair[ index0 ];
        Vec3f v1 = normalsHair[ index1 ];
        Vec3f v2 = normalsHair[ index2 ];
        triangleNormals.push_back((v0+v1+v2)/3.f);
    }
}