#include "vector.h"

namespace modGeo {

	vector::vector(void) { }

	vector::vector(double x, double y, double z) {
		tab[0] = x;
		tab[1] = y;
		tab[2] = z;
		tab[3] = 1.0;
	}

	vector::vector(ci::Vec3f v) {
		tab[0] = v.x;
		tab[1] = v.y;
		tab[2] = v.z;
		tab[3] = 1.0;
	}

	vector::~vector(void) { }

	ci::Vec3f vector::toVec3f() {
		return ci::Vec3f((float)tab[0],(float)tab[1],(float)tab[2]);
	}

	void vector::zero() {
		tab[0] = tab[1] = tab[2] = 0.0;
		tab[3] = 1.0;
	}

	void vector::fix() {
		tab[0] /= tab[3];
		tab[1] /= tab[3];
		tab[2] /= tab[3];
		tab[3] /= tab[3];
	}

	void vector::normalize() {
		double d = length();
		tab[0] /= d;
		tab[1] /= d;
		tab[2] /= d;
	}

	vector oneThird(const vector &v1, const vector &v2) {
		return v1 + (v2 - v1) * (1.0 / 3.0);
	}

	vector midPoint(const vector &v1, const vector &v2) {
		return  v1 + (v2 - v1) * 0.5;
	}


	double dot(const vector &v1, const vector &v2) {
		return v1.tab[0] * v2.tab[0] + v1.tab[1] * v2.tab[1] + v1.tab[2] * v2.tab[2];
	}

	vector cross(const vector &a, const vector &b) {
		return vector(a[1]*b[2]-a[2]*b[1],a[2]*b[0]-a[0]*b[2],a[0]*b[1]-a[1]*b[0]);
	}


	double distance(const vector &v1, const vector &v2) {
		return (v2-v1).length();
		//double dx = v2.tab[0] - v1.tab[0];
		//double dy = v2.tab[1] - v1.tab[1];
		//double dz = v2.tab[2] - v1.tab[2];
		//return sqrt(dx*dx+dy*dy+dz*dz);
	}

	double vector::length() {
		return sqrt(dot(*this,*this));
	}

	double vector::length2() {
		return dot(*this,*this);
	}

	double& vector::operator[](int i) {
		return this->tab[i];
	}

	const double& vector::operator[](int i) const {
		return this->tab[i];
	}

	std::ostream& operator<<(std::ostream &out, const vector &vec) {
		out << "v: ";
		for(int i = 0; i < 4; ++i) {
			out << vec.tab[i] << " ";
		}
		out << std::endl;
		return out;
	}

	vector operator-(const vector &lhs, const vector &rhs) {
		return vector(lhs.tab[0] - rhs.tab[0], 
					  lhs.tab[1] - rhs.tab[1],
					  lhs.tab[2] - rhs.tab[2]);
	}

	vector operator+(const vector &lhs, const vector &rhs) {
		return vector(rhs.tab[0] + lhs.tab[0], 
					  rhs.tab[1] + lhs.tab[1],
					  rhs.tab[2] + lhs.tab[2]);
	}

	vector operator*(const vector &lhs, const double a) {
		return vector(a * lhs.tab[0], 
					  a * lhs.tab[1],
					  a * lhs.tab[2]);
	}

	vector operator/(const vector &lhs, const double a) {
		return vector(lhs.tab[0] / a, 
					  lhs.tab[1] / a,
					  lhs.tab[2] / a);
	}

	vector operator/(const vector &lhs, const vector &rhs) {
		return vector(lhs.tab[0] / rhs.tab[0], 
					  lhs.tab[1] / rhs.tab[1],
					  lhs.tab[2] / rhs.tab[2]);
	}

	vector operator*(const vector &lhs, const vector &rhs) {
		return vector(lhs.tab[0] * rhs.tab[0], 
					  lhs.tab[1] * rhs.tab[1],
					  lhs.tab[2] * rhs.tab[2]);
	}

	vector vector::operator-() const {
		return vector(-tab[0], -tab[1], -tab[2]);
	}

}
